/* 
   eigenplane - find basis vectors for eigenplanes of an isometry of S^3 represented as a 4x4 matrix

   Copyright (C) 2004 Boud Roukema

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

#include <termios.h>
#include <grp.h>
#include <pwd.h>
*/

#include <stdio.h>
#include <sys/types.h>

#include "system.h"

#ifdef HAVE_ARGP_H
#  include <argp.h>
#endif HAVE_ARGP_H


#define EXIT_FAILURE 1

#if ENABLE_NLS
# include <libintl.h>
# define _(Text) gettext (Text)
#else
# define textdomain(Domain)
# define _(Text) Text
#endif
#define N_(Text) Text

/* fortran main subroutine stuff 
 * e.g. 
 * info autoconf 
 * C-s F77_FUNC
 */
#define MAINSUB_F77 F77_FUNC(mainsub,MAINSUB)
#ifdef __cplusplus
extern "C"  /* prevent C++ name mangling */
#endif
void MAINSUB_F77(char *oname, int *nlen_oname, 
		 char *iname, int *nlen_iname, 
		 int *want_interactive,
		 int *want_random,
		 int *want_verbose,
		 int *want_warning_disable,
		 char *random_seed, int *nlen_random_seed,
		 char *precision, int *nlen_precision
		 );

char *xmalloc ();
char *xrealloc ();
char *xstrdup ();

#ifdef HAVE_ARGP_H
  static error_t parse_opt (int key, char *arg, struct argp_state *state);
  static void show_version (FILE *stream, struct argp_state *state);
#else
  static void show_version (FILE *stream);
#endif HAVE_ARGP_H

/* argp option keys */
enum {DUMMY_KEY=129
      ,DIRECTORY_KEY
};

/* Option flags and variables.  These are initialized in parse_opt.  */

char *oname;			/* --output=FILE */
FILE *ofile;
char *iname;			/* FILE   (for input) */
FILE *ifile;
int want_interactive;		/* --interactive */
int want_random;		/* --random */
int want_verbose;		/* --verbose */
int want_warning_disable;	/* --warning-disable */
char *random_seed;	        /* --random-seed= */
char *precision;	        /* --precision= */


#ifdef HAVE_ARGP_H
static struct argp_option options[] =
{
  /*  { "interactive", 'i',           NULL,            0,
      N_("(not presently used)"), 0 },  */
  { "random", 'r',           NULL,            0,
    N_("generate a Random isometry in a Random orthonormal basis"), 0 },
  { "output",      'o',           N_("FILE"),      0,
    N_("send Output to FILE instead of standard output"), 0 },
  { "verbose",     'v',           NULL,            0,
    N_("print more Verbal information"), 0 },
  { "warning-disable",      'w',           NULL,      0,
    N_("disable printing Warnings"), 0 },
  { "random-seed",      's',      N_("random seed"),      0,
    N_("provide an integer Seed for random generator"), 0 },
  { "precision",      'p',      N_("precision"),      0,
    N_("angular Precision with which to find the angles (not guaranteed)"), 0 },
  { NULL, 0, NULL, 0, NULL, 0 }
};

/* The argp functions examine these global variables.  */
const char *argp_program_bug_address = "<boud at astro.uni.torun.pl>";
void (*argp_program_version_hook) (FILE *, struct argp_state *) = show_version;


static struct argp argp =
{
  options, parse_opt, N_("[FILE]"),
  N_("find basis vectors for eigenplanes of S^3 isometry as 4x4 matrix in FILE\nWith no FILE, read standard input in fortran compatible format"),
  NULL, NULL, NULL
};
#endif HAVE_ARGP_H

int
/* main (int argc, char **argv) */
/* F77_MAIN */
main (int argc, char **argv)
{
  int  nlen_oname, nlen_iname, nlen_dir=0; 
  int  nlen_random_seed; 
  int  nlen_precision; 

  textdomain(PACKAGE);


  /* Set up default values.  */
  oname = "stdout";
  ofile = stdout;
  iname = "stdin";
  ifile = stdin;
  want_interactive = 0;
  want_random = 0;
  want_verbose = 0;
  want_warning_disable = 0;
  random_seed = "none";    /* default set in mainsub.f */
  precision = "none";      /* default set in mainsub.f */
  iname = "stdin";

#if HAVE_ARGP_H
  argp_parse(&argp, argc, argv, 0, NULL, NULL);
#else
  printf("Warning: no command line options since a recent glibc is not installed.\n");
#endif HAVE_ARGP_H


  /* TODO: do the work */

  nlen_oname= strlen(oname);
  /*     printf("oname = __%s__ \n nlen_oname= %d\n",oname,nlen_oname);  */
  nlen_iname= strlen(iname);
  nlen_random_seed= strlen(random_seed);
  nlen_precision= strlen(precision);

  MAINSUB_F77(oname, &nlen_oname,
	      iname, &nlen_iname,
	      &want_interactive, 
	      &want_random,
	      &want_verbose, 
	      &want_warning_disable, 
	      random_seed, &nlen_random_seed,
	      precision, &nlen_precision
	      );

  exit (0);
}

#ifdef HAVE_ARGP_H
/* Parse a single option.  */
static error_t
parse_opt (int key, char *arg, struct argp_state *state)
{
  switch (key)
    {
    case ARGP_KEY_INIT:
      break;

    case 'i':			/* --interactive */
      want_interactive = 1;
      break;
    case 'r':			/* --interactive */
      want_random = 1;
      break;
    case 'o':			/* --output */
      oname = xstrdup (arg);
      ofile = fopen (oname, "w");
      if (!ofile)
	{
	argp_failure (state, EXIT_FAILURE, errno,
		      _("Cannot open %s for writing"), oname);
	}
      else{
	fclose(ofile);  /* added post-autoproject */
      };
      break;
    case 'v':			/* --verbose */
      want_verbose = 1;
      break;
    case 'w':			/* --warning-disable */
      want_warning_disable = 1;
      break;
    case 's':                   /* --random-seed */
      random_seed = xstrdup(arg);
      break;
    case 'p':                   /* --precision */
      precision = xstrdup(arg);
      break;
    case ARGP_KEY_ARG:		/* [FILE]... */
      iname = xstrdup (arg);
      ifile = fopen (iname, "r");
      if (!ifile)
	{
	argp_failure (state, EXIT_FAILURE, errno,
		      _("Cannot open %s for reading"), iname);
	}
      else{
	fclose(ifile);  /* added post-autoproject */
      };
      break;

      /*      printf("filename %s ignored for this version \n",xstrdup(arg));*/
      /* TODO: Do something with ARG, or remove this case and make
         main give argp_parse a non-NULL fifth argument.  */
      /* break;  */
    default:
      return ARGP_ERR_UNKNOWN;
    }
  return 0;
}
#endif HAVE_ARGP_H


/* Show the version number and copyright information.  */
static void
#ifdef HAVE_ARGP_H
  show_version (FILE *stream, struct argp_state *state)
#else
  show_version (FILE *stream)
#endif HAVE_ARGP_H
{
#ifdef HAVE_ARGP_H
  (void) state;
#endif HAVE_ARGP_H
  /* Print in small parts whose localizations can hopefully be copied
     from other programs.  */
  fputs(PACKAGE" "VERSION"\n", stream);
  fprintf(stream, _("Written by %s.\n\n"), "Boud Roukema");
  fprintf(stream, _("Copyright (C) %s %s\n"), "2004", "Boud Roukema");
  fputs(_("\
This program is free software; you may redistribute it under the terms of\n\
the GNU General Public License.  This program has absolutely no warranty.\n"),
	stream);
}
